#include "sprox_desfire_i.h"

#include <stdio.h>
#include <string.h>

SPROX_DESFIRE_LIB const TCHAR *SPROX_DESFIRE_API DesFire_GetErrorMessage(SWORD status)
{
  switch (status)
  {
    case DF_OPERATION_OK:
      return _T("function was executed without failure");

    case DFCARD_ERROR:
      return _T("DesFire : unknown error");
    case DFCARD_ERROR - DF_NO_CHANGES:
      return _T("DesFire : no changes don to backup file, no need to commit/abort");
    case DFCARD_ERROR - DF_OUT_OF_EEPROM_ERROR:
      return _T("DesFire : insufficient NV-memory to complete command");
    case DFCARD_ERROR - DF_ILLEGAL_COMMAND_CODE:
      return _T("DesFire : command code not supported");
    case DFCARD_ERROR - DF_INTEGRITY_ERROR:
      return _T("DesFire : CRC or MAC does not match, or invalid padding bytes");
    case DFCARD_ERROR - DF_NO_SUCH_KEY:
      return _T("DesFire : invalid key number specified");
    case DFCARD_ERROR - DF_LENGTH_ERROR:
      return _T("DesFire : length of command string invalid");
    case DFCARD_ERROR - DF_PERMISSION_DENIED:
      return _T("DesFire : current configuration or status does not allow the requested command");
    case DFCARD_ERROR - DF_PARAMETER_ERROR:
      return _T("DesFire : value of the parameter(s) invalid");
    case DFCARD_ERROR - DF_APPLICATION_NOT_FOUND:
      return _T("DesFire : requested application not present on the card");
    case DFCARD_ERROR - DF_APPL_INTEGRITY_ERROR:
      return _T("DesFire : unrecoverable error within application, application will be disabled");
    case DFCARD_ERROR - DF_AUTHENTICATION_CORRECT:
      return _T("DesFire : successfull authentication");
    case DFCARD_ERROR - DF_AUTHENTICATION_ERROR:
      return _T("DesFire : current authentication status does not allow the requested command");
    case DFCARD_ERROR - DF_ADDITIONAL_FRAME:
      return _T("DesFire : additionnal data frame is expecred to be sent");
    case DFCARD_ERROR - DF_BOUNDARY_ERROR:
      return _T("DesFire : attempt to read or write data out of the file's or record's limits");
    case DFCARD_ERROR - DF_CARD_INTEGRITY_ERROR:
      return _T("DesFire : unrecoverable error within the card, the card will be disabled");
    case DFCARD_ERROR - DF_COMMAND_ABORTED:
      return _T("DesFire : the current command has been aborded");
    case DFCARD_ERROR - DF_CARD_DISABLED_ERROR:
      return _T("DesFire : card was disabled by an unrecoverable error");
    case DFCARD_ERROR - DF_COUNT_ERROR:
      return _T("DesFire : maximum number of 28 applications has been reached");
    case DFCARD_ERROR - DF_DUPLICATE_ERROR:
      return _T("DesFire : the specified file or application already exists");
    case DFCARD_ERROR - DF_FILE_NOT_FOUND:
      return _T("DesFire : the specified file does not exists");
    case DFCARD_ERROR - DF_FILE_INTEGRITY_ERROR:
      return _T("DesFire : unrecoverable error within file, file will be disabled");
    case DFCARD_ERROR - DF_EEPROM_ERROR:
      return _T("DesFire : could not complete NV-memory write operation, due to power loss, aborting");

    case DFCARD_LIB_CALL_ERROR:
      return _T("DesFire : invalid parameters in function call");
    case DFCARD_OUT_OF_MEMORY:
      return _T("DesFire : not enough memory");
    case DFCARD_OVERFLOW:
      return _T("DesFire : supplied buffer is too short");
    case DFCARD_WRONG_KEY:
      return _T("DesFire : wrong DES/3DES key");
    case DFCARD_WRONG_MAC:
      return _T("DesFire : wrong MAC in incoming data");
    case DFCARD_WRONG_CRC:
      return _T("DesFire : wrong CRC in incoming data");
    case DFCARD_WRONG_LENGTH:
      return _T("DesFire : wrong length of incoming data");
    case DFCARD_WRONG_PADDING:
      return _T("DesFire : wrong padding in incoming data");
    case DFCARD_WRONG_FILE_TYPE:
      return _T("DesFire : wrong file type");
    case DFCARD_WRONG_RECORD_SIZE:
      return _T("DesFire : wrong record size");
  }

  return SPROX_GetErrorMessage(status);
}

SPROX_DESFIRE_LIB SWORD SPROX_DESFIRE_API DesFireCard_ExplainDataFileSettings(BYTE * additionnal_settings_array, DWORD * eFileSize)
{
  DF_ADDITIONAL_FILE_SETTINGS *p = (DF_ADDITIONAL_FILE_SETTINGS *) additionnal_settings_array;

  if (p == NULL)
    return DFCARD_LIB_CALL_ERROR;

  if (eFileSize != NULL)
    *eFileSize = p->stDataFileSettings.eFileSize;

  return DF_OPERATION_OK;
}

SPROX_DESFIRE_LIB SWORD SPROX_DESFIRE_API DesFireCard_ExplainValueFileSettings(BYTE * additionnal_settings_array, SDWORD * lLowerLimit, SDWORD * lUpperLimit,
                                                                               DWORD * eLimitedCredit, BYTE * bLimitedCreditEnabled)
{
  DF_ADDITIONAL_FILE_SETTINGS *p = (DF_ADDITIONAL_FILE_SETTINGS *) additionnal_settings_array;

  if (p == NULL)
    return DFCARD_LIB_CALL_ERROR;

  if (lLowerLimit != NULL)
    *lLowerLimit = p->stValueFileSettings.lLowerLimit;
  if (lUpperLimit != NULL)
    *lUpperLimit = p->stValueFileSettings.lUpperLimit;
  if (eLimitedCredit != NULL)
    *eLimitedCredit = p->stValueFileSettings.eLimitedCredit;
  if (bLimitedCreditEnabled != NULL)
    *bLimitedCreditEnabled = p->stValueFileSettings.bLimitedCreditEnabled;

  return DF_OPERATION_OK;
}

SPROX_DESFIRE_LIB SWORD SPROX_DESFIRE_API DesFireCard_ExplainRecordFileSettings(BYTE * additionnal_settings_array, DWORD * eRecordSize, DWORD * eMaxNRecords,
                                                                                DWORD * eCurrNRecords)
{
  DF_ADDITIONAL_FILE_SETTINGS *p = (DF_ADDITIONAL_FILE_SETTINGS *) additionnal_settings_array;

  if (p == NULL)
    return DFCARD_LIB_CALL_ERROR;

  if (eRecordSize != NULL)
    *eRecordSize = p->stRecordFileSettings.eRecordSize;
  if (eMaxNRecords != NULL)
    *eMaxNRecords = p->stRecordFileSettings.eMaxNRecords;
  if (eCurrNRecords != NULL)
    *eCurrNRecords = p->stRecordFileSettings.eCurrNRecords;

  return DF_OPERATION_OK;
}
