#include "cardappz.h"
#include <assert.h>
#include <stdio.h>
#include <conio.h>

/* Those static data are the default values used to create contracts */

/* ----------------------------------------------------------------- */
static unsigned char Contract_Network[3] = { 0xAB, 0x12, 0x34 };
static unsigned char Contract_ProviderID = 0xDD;
static unsigned char Contract_Mode = 1;
static unsigned char Contract_Counter = 0;
static unsigned char Contract_Areas = 0x1F;
static unsigned char Contract_Class = 2;
static unsigned char Contract_ValidWeekDays = 0x7F;
static unsigned char Contract_ValidTotalDays = 0;
static unsigned char Contract_StartDate[4] = { 0, 0, 0, 0 };
static unsigned char Contract_EndDate[4] = { 0, 0, 0, 23 };
static unsigned long Counter_Value;

/*
 * TransportDisplayEvent
 * ---------------------
 * Display one event
 */
int TransportDisplayEvent(CARDAPPZ_CTX_ST * ctx, TRANSPORT_EVENT_ST * event)
{
  assert(ctx != NULL);
  assert(event != NULL);

  printf("Timestamp      : %02d/%02d/%02d %02d:%02d\n", event->TimeStamp_Day,
         event->TimeStamp_Month, event->TimeStamp_Year,
         event->TimeStamp_Hour, event->TimeStamp_Min);
  printf("Network        : %02X%02X%02X\n", event->Network[0],
         event->Network[1], event->Network[2]);
  printf("Contract       : %d\n", event->Contract);
  printf("Counter        : %d\n", event->Counter);
  printf("Charged        : %d\n", event->Charged);
  printf("Action         : %d\n", event->Action);
  printf("Location       : %02X%02X%02X\n", event->Location[0],
         event->Location[1], event->Location[2]);
  printf("RemainingDays  : %d\n", event->RemainingDays);

  pause();
  return CARDAPPZ_OK;
}


/*
 * TransportListEvents
 * -------------------
 * Display all the events currently in the card
 */
int TransportListEvents(CARDAPPZ_CTX_ST * ctx)
{
  TRANSPORT_EVENT_ST event;
  int rc, i;

  assert(ctx != NULL);

  for (i = 0;; i++)
  {
    rc = SelectTransport(ctx);
    if (rc)
      break;

    rc = ReadTransportEvent(ctx, i, &event);
    if (rc)
      break;

    printf("*** Event %d ***\n", i + 1);
    TransportDisplayEvent(ctx, &event);
  }

  if (i == 0)
  {
    printf("No event, or fatal card error\n");
  } else
  {
    printf("%d event(s) found\n", i);
  }

  pause();
  return CARDAPPZ_OK;
}

/*
 * TransportDisplayContract
 * ------------------------
 * Display one contract
 */
int TransportDisplayContract(CARDAPPZ_CTX_ST * ctx,
                             TRANSPORT_CONTRACT_ST * contract)
{
  int i, rc;

  assert(ctx != NULL);
  assert(contract != NULL);

  printf("Priority       : %d\n", contract->Priority);
  printf("Network        : %02X%02X%02X\n", contract->Network[0],
         contract->Network[1], contract->Network[2]);
  printf("ProviderId     : %02X\n", contract->ProviderID);
  printf("CustomerID     : %02X%02X%02X%02X\n", contract->CustomerID[0],
         contract->CustomerID[1], contract->CustomerID[2],
         contract->CustomerID[3]);
  printf("Passengers     : %d\n", contract->Passengers);
  printf("Mode           : %d ", contract->Mode);
  switch (contract->Mode)
  {
    case 1:
      printf("(Pass)");
      break;
    case 2:
      printf("(Tickets)");
      break;
    case 3:
      printf("(Coupons)");
      break;
  }
  printf("\n");
  printf("Areas          : ");
  for (i = 0; i < 8; i++)
  {
    if (contract->Areas & (1 << i))
      printf("%d", i + 1);
    else
      printf("-");
  }
  printf("\n");
  printf("Class          : %d\n", contract->Class);
  printf("Counter        : %d\n", contract->Counter);
  printf("ValidWeekDays  : ");
  for (i = 0; i < 7; i++)
  {
    if (contract->ValidWeekDays & (1 << i))
      printf("%d", i + 1);
    else
      printf("-");
  }
  printf("\n");
  printf("ValidTotalDays : %d\n", contract->ValidTotalDays);
  printf("StartDate      : %02d/%02d/%02d %02dh00\n",
         contract->StartDate_Day, contract->StartDate_Month,
         contract->StartDate_Year, contract->StartDate_Hour);
  printf("EndDate        : %02d/%02d/%02d %02dh59\n", contract->EndDate_Day,
         contract->EndDate_Month, contract->EndDate_Year,
         contract->EndDate_Hour);

  /* Verify the signature (the signature key is not diversified) */
  rc =
    TransportVerifyContractSignature(ctx, contract, TRANSPORT_SIGNATURE_KEY);
  if (rc == CARDAPPZ_OK)
  {
    printf("Signature      : Valid\n");
  } else if (rc == CARDAPPZ_CARD_SIGNATURE_ERROR)
  {
    printf("Signature      : Invalid (card is forged)\n");
  } else
  {
    printf("Failed to verify contract signature\n");
  }

  pause();
  return CARDAPPZ_OK;
}

/*
 * TransportListContracts
 * ----------------------
 * Display all the contracts currently in the card
 */
int TransportListContracts(CARDAPPZ_CTX_ST * ctx)
{
  TRANSPORT_CONTRACT_ST contract;
  int rc, i;

  assert(ctx != NULL);

  for (i = 0;; i++)
  {
    rc = SelectTransport(ctx);
    if (rc)
      break;

    rc = ReadTransportContract(ctx, i, &contract);
    if (rc)
      break;

    printf("*** Contract %d ***\n", i + 1);
    TransportDisplayContract(ctx, &contract);
  }

  if (i == 0)
  {
    printf("No contract, or fatal card error\n");
  } else
  {
    printf("%d contract(s) found\n", i);
  }

  pause();
  return CARDAPPZ_OK;
}

/*
 * TransportDefineContract
 * -----------------------
 * Let user enter contract data
 */
int TransportDefineContract(CARDAPPZ_CTX_ST * ctx,
                            TRANSPORT_CONTRACT_ST * contract,
                            unsigned long *counter)
{
  char buffer[256];
  time_t tt;
  struct tm *ptm, wtm;
  size_t i, t;
  char *p;

  assert(ctx != NULL);
  assert(contract != NULL);
  assert(counter != NULL);

  /* Load current defaults into contract */
  /* ----------------------------------- */

  memcpy(contract->Network, Contract_Network, 3);
  contract->ProviderID = Contract_ProviderID;
  if (ctx->card_type & CARD_TYPE_14443_A)
  {
    memcpy(contract->CustomerID, ctx->card_ident.a.snr, 4);
  } else if (ctx->card_type & CARD_TYPE_14443_B)
  {
    memcpy(contract->CustomerID, ctx->card_ident.b.atq, 4);
  } else
    return CARDAPPZ_EXCEPTION;

  contract->Passengers = 1;
  contract->Mode = Contract_Mode;
  contract->Counter = Contract_Counter;
  contract->Areas = Contract_Areas;
  contract->Class = Contract_Class;
  contract->ValidWeekDays = Contract_ValidWeekDays;
  contract->ValidTotalDays = Contract_ValidTotalDays;
  contract->StartDate_Year = Contract_StartDate[0];
  contract->StartDate_Month = Contract_StartDate[1];
  contract->StartDate_Day = Contract_StartDate[2];
  contract->StartDate_Hour = Contract_StartDate[4];
  contract->EndDate_Year = Contract_EndDate[0];
  contract->EndDate_Month = Contract_EndDate[1];
  contract->EndDate_Day = Contract_EndDate[2];
  contract->EndDate_Hour = Contract_EndDate[4];

  *counter = Counter_Value;

  /* Ask user to populate contract fields */
  /* ------------------------------------ */

  for (;;)
  {
    printf("*** Define new transport contract ***\n");

    printf("Priority [%d] ?", contract->Priority);
    gets(buffer);
    if (strlen(buffer))
      contract->Priority = atoi(buffer);

    printf("Network (hex value) [%02X%02X%02X] ?", contract->Network[0],
           contract->Network[1], contract->Network[2]);
    gets(buffer);
    if (strlen(buffer))
      gethex(buffer, contract->Network, 3);

    printf("Provider (hex value) [%02X] ?", contract->ProviderID);
    gets(buffer);
    if (strlen(buffer))
      gethex(buffer, &contract->ProviderID, 1);

    printf("Customer ID (hex value) [%02X%02X%02X%02X] ?",
           contract->CustomerID[0], contract->CustomerID[1],
           contract->CustomerID[2], contract->CustomerID[3]);
    gets(buffer);
    if (strlen(buffer))
      gethex(buffer, contract->CustomerID, 4);

    printf("Passengers [%d] ?", contract->Passengers);
    gets(buffer);
    if (strlen(buffer))
      contract->Passengers = atoi(buffer);

    for (;;)
    {
      printf("Mode (1=Pass, 2=Tickets, 3=Coupons) [%d] ?", contract->Mode);
      gets(buffer);
      if (strlen(buffer))
        contract->Mode = atoi(buffer);
      if ((contract->Mode >= 1) && (contract->Mode <= 3))
        break;
    }

    if (contract->Mode != 1)
    {
      for (;;)
      {
        printf("Counter to be used [%d] ?", contract->Counter);
        gets(buffer);
        if (strlen(buffer))
          contract->Counter = atoi(buffer);
        if (contract->Counter >= 1)
          break;
      }
    } else
      contract->Counter = 0;

    if (contract->Mode == 2)
    {
      printf("Number of tickets [%ld] ?", *counter);
      gets(buffer);
      if (strlen(buffer))
        *counter = atoi(buffer);
    } else if (contract->Mode == 3)
    {
      printf("Number of coupons [%ld] ?", *counter);
      gets(buffer);
      if (strlen(buffer))
        *counter = atoi(buffer);
    } else
      *counter = 0;

    printf("Valid areas [");
    for (i = 0; i < 8; i++)
    {
      if (contract->Areas & (1 << i))
        printf("%d", i + 1);
      else
        printf("-");
    }
    printf("] ");
    gets(buffer);
    if (strlen(buffer))
    {
      contract->Areas = 0;
      for (i = 0; i < strlen(buffer); i++)
      {
        t = buffer[i];
        if ((t >= '1') && (t <= '8'))
        {
          t -= '1';
          contract->Areas |= (1 << t);
        }
      }
    }

    printf("Valid week days (1=monday, 7=sunday) [");
    for (i = 0; i < 7; i++)
    {
      if (contract->ValidWeekDays & (1 << i))
        printf("%d", i + 1);
      else
        printf("-");
    }
    printf("] ");
    gets(buffer);
    if (strlen(buffer))
    {
      contract->ValidWeekDays = 0;
      for (i = 0; i < strlen(buffer); i++)
      {
        t = buffer[i];
        if ((t >= '1') && (t <= '7'))
        {
          t -= '1';
          contract->ValidWeekDays |= (1 << t);
        }
      }
    }

    printf("Total number of days (0=unlimited) [%d] ?",
           contract->ValidTotalDays);
    gets(buffer);
    if (strlen(buffer))
      contract->ValidTotalDays = atoi(buffer);

    /* Get first day of month */
    tt = time(NULL);
    ptm = localtime(&tt);
    if (ptm != NULL)
      wtm = *ptm;
    wtm.tm_mday = 1;
    tt = mktime(&wtm);
    ptm = localtime(&tt);
    if (ptm != NULL)
      wtm = *ptm;

    if (contract->StartDate_Year == 0)
      contract->StartDate_Year = wtm.tm_year % 100;
    if ((contract->StartDate_Month == 0) || (contract->StartDate_Month > 12))
      contract->StartDate_Month = wtm.tm_mon + 1;
    if ((contract->StartDate_Day == 0) || (contract->StartDate_Day > 31))
      contract->StartDate_Day = 1;
    if (contract->StartDate_Hour > 23)
      contract->StartDate_Hour = 23;

    printf("First day of validity [%02d/%02d/%02d %02dh00] ?",
           contract->StartDate_Day, contract->StartDate_Month,
           contract->StartDate_Year, contract->StartDate_Hour);
    gets(buffer);
    if (strlen(buffer))
    {
      p = buffer;
      contract->StartDate_Day = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->StartDate_Month = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->StartDate_Year = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->StartDate_Hour = atoi(p);
    }

    /* Get first day of next month */
    wtm.tm_mon++;
    tt = mktime(&wtm);
    ptm = localtime(&tt);
    if (ptm != NULL)
      wtm = *ptm;

    if (contract->EndDate_Year == 0)
      contract->EndDate_Year = wtm.tm_year % 100;
    if ((contract->EndDate_Month == 0) || (contract->EndDate_Month > 12))
      contract->EndDate_Month = wtm.tm_mon + 1;
    if ((contract->EndDate_Day == 0) || (contract->EndDate_Day > 31))
      contract->EndDate_Day = 1;
    if (contract->EndDate_Hour > 23)
      contract->EndDate_Hour = 23;

    printf("Last day of validity [%02d/%02d/%02d %02dh59] ?",
           contract->EndDate_Day, contract->EndDate_Month,
           contract->EndDate_Year, contract->EndDate_Hour);
    gets(buffer);
    if (strlen(buffer))
    {
      p = buffer;
      contract->EndDate_Day = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->EndDate_Month = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->EndDate_Year = atoi(p);
      while ((*p != '/') && (*p != ' ') && (*p != '\0'))
        p++;
      contract->EndDate_Hour = atoi(p);
    }

    printf("\n");
    if (confirm("Save current values", 1))
      break;
  }

  /* Copy contract values into default */
  /* --------------------------------- */

  Contract_Mode = contract->Mode;
  Contract_Counter = contract->Counter;
  Contract_Areas = contract->Areas;
  Contract_Class = contract->Class;
  Contract_ValidWeekDays = contract->ValidWeekDays;
  Contract_ValidTotalDays = contract->ValidTotalDays;
  Contract_StartDate[0] = contract->StartDate_Year;
  Contract_StartDate[1] = contract->StartDate_Month;
  Contract_StartDate[2] = contract->StartDate_Day;
  Contract_StartDate[4] = contract->StartDate_Hour;
  Contract_EndDate[0] = contract->EndDate_Year;
  Contract_EndDate[1] = contract->EndDate_Month;
  Contract_EndDate[2] = contract->EndDate_Day;
  Contract_EndDate[4] = contract->EndDate_Hour;

  Counter_Value = *counter;

  /* Done */
  return CARDAPPZ_OK;
}

/*
 * TransportAddContract
 * --------------------
 * Ask user to populate a new contract, and append it to the card
 */
int TransportAddContract(CARDAPPZ_CTX_ST * ctx)
{
  TRANSPORT_CONTRACT_ST contract = { 0 };
  unsigned long counter = 0;
  int rc;

  /* Ask user to enter contract data */
  printf("*** Please enter parameters for contract ***\n");
  rc = TransportDefineContract(ctx, &contract, &counter);
  if (rc)
  {
    printf("Failed\n");
    return CARDAPPZ_OK;
  }

  /* Compute the signature (the signature key is not diversified) */
  rc =
    TransportBuildContractSignature(ctx, &contract, TRANSPORT_SIGNATURE_KEY);
  if (rc)
  {
    printf("Failed to compute the signature\n");
    return CARDAPPZ_OK;
  }

  /* Ready to write... */
  if (!confirm("Write contract values into the card", 1))
    return CARDAPPZ_OK;

  /* Now writing */
  printf("*** Writing new contract into transport application ***\n");
  rc = AppendTransportContract(ctx, &contract, counter);
  if (rc)
    printf("Failed !\n");
  else
    printf("Success\n");

  pause();
  return CARDAPPZ_OK;
}

/*
 * TransportRemoveContracts
 * ------------------------
 * Remove all contracts from the card
 */
int TransportRemoveContracts(CARDAPPZ_CTX_ST * ctx)
{
  int rc;

  if (!confirm("Erase all contracts", 0))
    return CARDAPPZ_OK;

  printf("*** Erasing contracts from transport application ***\n");
  rc = EraseTransportContracts(ctx);
  if (rc)
    printf("Failed !\n");
  else
    printf("Success\n");

  pause();
  return CARDAPPZ_OK;
}

/*
 * TransportMenu
 * -------------
 * Menu of the transport application
 */
int TransportMenu(CARDAPPZ_CTX_ST * ctx)
{
  int rc;
  int c;

  for (;;)
  {
    printf("\n");
    printf("***** TRANSPORT MENU *****\n");
    printf("Personalisation :\n");
    printf("L -- (L)ist contracts\n");
    printf("A -- (A)dd a new contract\n");
    printf("R -- (R)emove all contracts\n");
    printf("Control :\n");
    printf("L -- (L)ist contracts\n");
    printf("E -- List last (E)vents\n");
    printf("C -- On-the-field (C)ontrol\n");
    printf("End-user :\n");
    printf("W -- Enter sub(W)ay\n");
    printf("T -- Take the (T)rain\n");
    printf("I -- Take a taxi, Check-(I)n\n");
    printf("O -- Leave the taxi, Pay-(O)ut\n");
    printf("\n");
    printf("X -- E(X)it - Back to main menu\n");

    c = toupper(getch());
    switch (c)
    {
      case 'L':
        rc = TransportListContracts(ctx);
        break;
      case 'A':
        rc = TransportAddContract(ctx);
        break;
      case 'R':
        rc = TransportRemoveContracts(ctx);
        break;
      case 'E':
        rc = TransportListEvents(ctx);
        break;
      case 'C':
        rc = TransportControl(ctx);
        break;
      case 'W':
        rc = TransportSubway(ctx);
        break;
      case 'T':
        rc = TransportTrain(ctx);
        break;
      case 'I':
        rc = TransportTaxiIn(ctx);
        break;
      case 'O':
        rc = TransportTaxiOut(ctx);
        break;
      case 'X':
        return CARDAPPZ_OK;
    }
  }

  return CARDAPPZ_OK;
}
