#include "springprox.h"

#include <stdio.h>
#include <string.h>

#include "fingertk.h"
#include "sagem_mso_ilv.h"

BYTE verify_ex(FINGERTK_DATA_ST *data);
void verify_async_action(BYTE code);

/* Perform verify */
/* -------------- */
void verify(FINGERTK_DATA_ST *data)
{
  printf("Verifiying...\n\n");

  switch (verify_ex(data))
  {
    case 2  : printf("\n\tPASSED, finger has been recognized\n");
              break;
    case 1  : printf("\n\tFAILED, finger has not been recognized\n");
              break;
    default : printf("\n\tERROR, verification failed\n");
  }

  printf("\n");
}


/* SAGEM MSO verify command (1 or 2 fingers) */
/* ----------------------------------------- */
/* return : 0 if verification has not been done (error ?)   */
/*          1 if verification failed (not the right user ?) */
/*          2 if verification succeeded (the right user !)  */
BYTE verify_ex(FINGERTK_DATA_ST *data)
{
  BYTE  buffer[1024];
  BYTE  async;
  WORD  size;
  WORD  length;
  SWORD rc;

  /* Build the ILV verify command               */
  /* (SEE SAGEM DOCUMENTATION FOR EXPLANATIONS) */
  /* ------------------------------------------ */

  memset(buffer, 0, sizeof(buffer));
  length = 0;

	/* Initialize ILV */
	ilv_init(buffer, &length, ILV_VERIFY);

  /* Build ILV */
  ilv_add_word(buffer, &length, 30);     /* Timeout = 60 seconds */
  ilv_add_word(buffer, &length, 5);      /* Matching threshold = 5 (recommended value) */
  ilv_add_byte(buffer, &length, 0);      /* Acquisition quality threshold = default (0) */

  if (data->finger1_len)
  {
    /* Add first finger */
    ilv_add_byte(buffer, &length, data->finger1_type);
    ilv_add_word(buffer, &length, data->finger1_len);
    ilv_add_data(buffer, &length, data->finger1_data, data->finger1_len);
  }

  if (data->finger2_len)
  {
    /* Add second finger */
    ilv_add_byte(buffer, &length, data->finger2_type);
    ilv_add_word(buffer, &length, data->finger2_len);
    ilv_add_data(buffer, &length, data->finger2_data, data->finger2_len);
  }

  /* We want to receive asynchronous events */
  ilv_add_byte(buffer, &length, ID_ASYNCHRONOUS_EVENT);
  ilv_add_word(buffer, &length, 4);
  ilv_add_dword(buffer, &length, 0x00000001);

  /* Write length in the ILV */
	ilv_final(buffer, &length);

  /* Send the command to the MSO and process its responses (including asynchronous messages) */
  /* --------------------------------------------------------------------------------------- */
  async = FALSE;
  for (;;)
  {
    size = sizeof(buffer);

    /* NOTE : the timeout of the exchange must be a little greater than the timeout offered  */
    /*        to the MSO, overwhise the library will return an error code before the MSO has */
    /*        finished is process                                                            */
    if (async)        
      rc = SPROX_MSO_Exchange(NULL, 0, buffer, &size, 35000, &async);
    else
      rc = SPROX_MSO_Exchange(buffer, length, buffer, &size, 35000, &async);

    if (rc != 0)
      return 0; /* Error */

    if (async)
    {
      /* Easy processing of asynchronous message, no error control */
      switch(buffer[4])
      {
        case 0x01 : verify_async_action(buffer[7]);
                    break;
      }

      /* NOTE : we MUST go back to MSO_Exchange AS SOON AS POSSIBLE because if we were to  */
      /*        miss any message (asynchronous or final result), the reader will report a  */
      /*        protocol error and abort the process                                       */
    } else
    {
      break; /* Last message -> end of the loop */
    }
  }

  /* 
   * Reply format is :
   *  0      : reply ID
   *  1 -> 2 : reply length
   *  3      : request status
   *  4      : matching result
   */

  /* Check returned reply and status */
  /* ------------------------------- */

  if (buffer[0] != ILV_VERIFY)
  {
    printf("Verify failed (MSO reply=0x%02X)\n", buffer[0]);
    return 0;
  }
  if (buffer[3] != 0x00)
  {
    printf("Verify failed (MSO request status=0x%02X)\n", buffer[3]);
    return 0;
  }

  /* Check matching result */
  /* --------------------- */

  switch (buffer[4])
  {
    case ILVSTS_HIT    : /* Verify -> OK ! */
                         return 2;

    case ILVSTS_NO_HIT : /* Verify -> KO ! */
                         return 1;
  }

  printf("Verify failed (MSO matching result=0x%02X)\n", buffer[4]);
  return 0;
}

void verify_async_action(BYTE code)
{
  printf("\t\t");

  switch (code)
  {
		case 0:
			printf("Place finger");
			break;
		case 1:
			printf("Move up");
			break;
		case 2:
			printf("Move down");
			break;
		case 3:
			printf("Move left");
			break;
		case 4:
			printf("Move right");
			break;
		case 5:
			printf("Press harder");
			break;
		case 6:
			printf("Move finger");
			break;
		case 7:
			printf("Remove finger");
			break;
		case 8:
			printf("Finger OK");
			break;
	}

  printf("\n");
}

