using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CodePPC.General;

using SpringProxAPI;

namespace SProxPollingTest
{
  public partial class fMain : Form
  {
    private TWorker worker = null;
    private ulong wOKCount = 0;
    private ulong wKOCount = 0;
  
    public fMain()
    {
      InitializeComponent();
    }

    private void fMain_Load(object sender, EventArgs e)
    {
      LoadSettings();      
      eLastErrorTxt.Text = "Click start !";
    }

    private void fMain_Closing(object sender, CancelEventArgs e)
    {
      SaveSettings();
    }

    private void LoadSettings()
    {
      string s = "\\Software\\Pro-Active\\SProxPolling";
      string t;

      Registry.CreateNewKey(Registry.HKEY_LOCAL_MACHINE, s);
      
      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "ReaderType");
      if (t == "PC/SC")
      {
        
      } else
      {
        
      }

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "14443a");
      cb14443a.Checked = (t == "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "14443b");
      cb14443b.Checked = (t == "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "14443b_mem");
      cb14443bMem.Checked = (t == "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "innovatron");
      cbInnovatron.Checked = (t == "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "always_on");
      rbRFFieldON.Checked = (t == "1");
      rbRFFieldOFF.Checked = (t != "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "interval");
      if (t == "0")   rbIntervalMini.Checked = true; else
      if (t == "100") rbInterval100.Checked = true; else
      if (t == "500") rbInterval500.Checked = true; else
                      rbInterval250.Checked = true;

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "reconnect");
      cbReconnectAuto.Checked = (t == "1");

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "no_sleep");
      cbNoSleep.Checked = (t == "1");
    }

    private void SaveSettings()
    {
      string s = "\\Software\\Pro-Active\\SProxPolling";
      string t;

      Registry.CreateNewKey(Registry.HKEY_LOCAL_MACHINE, s);

      t = Registry.ReadValue(Registry.HKEY_LOCAL_MACHINE, s, "ReaderType");
      if (t == "PC/SC")
      {

      }
      else
      {

      }

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "14443a", cb14443a.Checked ? "1" : "0");

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "14443b", cb14443b.Checked ? "1" : "0");

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "14443b_mem", cb14443bMem.Checked ? "1" : "0");

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "innovatron", cbInnovatron.Checked ? "1" : "0");

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "always_on", rbRFFieldON.Checked ? "1" : "0");

      if (rbIntervalMini.Checked) Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "interval", "0"); else
      if (rbInterval100.Checked)  Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "interval", "100"); else
      if (rbInterval500.Checked)  Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "interval", "500"); else
                                  Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "interval", "250");

      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "reconnect", cbReconnectAuto.Checked ? "1" : "0");
      
      Registry.WriteValue(Registry.HKEY_LOCAL_MACHINE, s, "no_sleep", cbNoSleep.Checked ? "1" : "0");      
    }

    private void fMain_LostFocus(object sender, EventArgs e)
    {
      SaveSettings();
    }

    private void tmrNoSleep_Tick(object sender, EventArgs e)
    {
      if (cbNoSleep.Checked)
      {
        Tools.SystemIdleTimerReset();
      }
    }
    
    private void WorkerDisplay(object sender, EventArgs e)
    {
      short wLastError = worker.LastError();
      eLastError.Text = String.Format("{0}", wLastError);
      
      if (wLastError == SPROX.MI_OK)
        eLastErrorTxt.Text = "";
      else
        eLastErrorTxt.Text = SPROX.GetErrorMessage(wLastError);
      
      eCardType.Text = worker.CardType();
      eCardUID.Text  = worker.CardUID();
      
      if (eCardType.Text != "")
      {
        wOKCount++;
        eCardOKCount.Text = String.Format("{0}", wOKCount);
      } else
      if (wLastError == SPROX.MI_NOTAGERR)
      {
        wKOCount++;
        eCardKOCount.Text = String.Format("{0}", wKOCount);      
      }
           
      if (worker.Terminated())
      {
        miEndPolling.Enabled = false;
        worker = null;       
        eLastErrorTxt.Text = "Fatal error, click start to resume";
        miStartPolling.Enabled = true;
      }

      Application.DoEvents();
    }
    
    public void WorkerCallback(object sender, EventArgs e)
    {     
      try
      {
        Invoke(new EventHandler(WorkerDisplay));
      } catch (Exception)
      {
      
      }    
    }

    private void miStartPolling_Click(object sender, EventArgs e)
    {
      ushort protos = 0;
      ushort interval = 250;
      
      if (cb14443a.Checked)     protos |= SPROX.PROTO_14443_A;
      if (cb14443b.Checked)     protos |= SPROX.PROTO_14443_B;
      if (cb14443bMem.Checked)  protos |= SPROX.PROTO_MEM_ASK|SPROX.PROTO_MEM_ST;
      if (cbInnovatron.Checked) protos |= SPROX.PROTO_14443_Bi;
      
      if (rbIntervalMini.Checked) interval = 0; else
      if (rbInterval100.Checked) interval = 100; else
      if (rbInterval500.Checked) interval = 500;

      miRFOFF.Enabled = false;
      miRFOFF.Checked = false;
      miStartPolling.Enabled = false;
      
      SaveSettings();

      eLastErrorTxt.Text = "Connecting to reader";
      
      worker = new TWorker();
      worker.Callback += new TWorker.TCallback(WorkerCallback);
      worker.Start(protos, interval, rbRFFieldOFF.Checked, cbReconnectAuto.Checked);

      miEndPolling.Enabled = true;
    }

    private void miEndPolling_Click(object sender, EventArgs e)
    {
      miEndPolling.Enabled = false;
      if (worker != null)
        worker.Stop();
      worker = null;
      
      eLastErrorTxt.Text = "Test terminated, click start !";
      
      miStartPolling.Enabled = true;
      miRFOFF.Enabled = true;
    }

    private void miExit_Click(object sender, EventArgs e)
    {      
      if (worker != null)
          worker.Stop();
      Close();
      Application.Exit();
    }

    private void miResetCounter_Click(object sender, EventArgs e)
    {
      wOKCount = 0;
      wKOCount = 0;
      eCardOKCount.Text = "";
      eCardKOCount.Text = "";
    }

    private void miRFOFF_Click(object sender, EventArgs e)
    {
      if (!miRFOFF.Enabled) return;
      if (miRFOFF.Checked)
      {
        miStartPolling.Enabled = false;
        
        SaveSettings();
        
        eLastErrorTxt.Text = "Running with RF OFF...";
        
        worker = new TWorker();
        worker.Callback += new TWorker.TCallback(WorkerCallback);
        worker.Start(cbReconnectAuto.Checked);

        
      } else
      {
        if (worker != null)
          worker.Stop();
        worker = null;
        
        eLastErrorTxt.Text = "Test terminated, click start !";
        
        miStartPolling.Enabled = true;
      }
    }
        
        
  }
}