using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CodePPC.General;

using System.Collections;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;

using SpringProxAPI;

namespace SProxPollingTest
{
  class TWorker
  {
    public delegate void TCallback(Object sender, EventArgs e);
    public event TCallback Callback;

    private Thread wThread;
    
    private bool wAllowed;
    private bool wRunning;
    
    private bool wConnected = false;
    private bool wFirstConnect = true;
    private bool wWasConnected = false;
    private short wLastError;
    
    private string wCardUID;
    private string wCardType;
    
    private ushort wInterval = 250;
    private ushort wProtos = SPROX.PROTO_14443_A|SPROX.PROTO_14443_B;
    private bool wStayOFF = false;
    private bool wStopRF = true;
    private bool wReconnect = false;
    
    public void Start(bool reconnect)
    {
      if (wRunning) return;
      
      wConnected = false;
      wWasConnected = false;
      
      wRunning = true;
      wAllowed = true;
      
      wProtos = 0;
      wInterval = 250;
      wStopRF = false;
      wReconnect = reconnect;
      
      wStayOFF = true;
           
      wThread = new Thread(new ThreadStart(wTask));
      wThread.Start();
    }
           
    public void Start(ushort protos, ushort interval, bool stop_rf, bool reconnect)
    {
      if (wRunning) return;
      
      wConnected = false;
      wWasConnected = false;
      
      wRunning = true;
      wAllowed = true;
      
      wProtos = protos;
      wInterval = interval;
      wStopRF = stop_rf;
      wReconnect = reconnect;
      
      if (wInterval == 0)
        wInterval = 10;
        
      wStayOFF = false;
      
      wThread = new Thread(new ThreadStart(wTask));
      wThread.Start();
    }

    public void Stop()
    {     
      wAllowed = false;     
    }
    
    public bool Terminated()
    {
      return !wRunning;
    }
    
    public short LastError()
    {
      return wLastError;
    }

    public string CardUID()
    {
      return wCardUID;
    }

    public string CardType()
    {
      return wCardType;
    }
    
    public void CardPresent(ushort proto, byte[] uid, byte uidlen)
    {
      byte i;
      
      wLastError = SPROX.MI_OK;
      
      switch (proto)
      {
        case SPROX.PROTO_14443_A  : wCardType = "ISO 14443-A (or NXP Mifare)"; break;
        case SPROX.PROTO_14443_B  : wCardType = "ISO 14443-B"; break;
        case SPROX.PROTO_14443_Bi : wCardType = "Calypso / Innovatron"; break;
        case SPROX.PROTO_15693    : wCardType = "ISO 15693"; break;
        case SPROX.PROTO_ICODE1   : wCardType = "NXP ICODE1"; break;
        case SPROX.PROTO_MEM_ASK  : wCardType = "Pseudo 14443-B (ASK CTS)"; break;
        case SPROX.PROTO_MEM_ST   : wCardType = "Pseudo 14443-B (ST SR/SRX)"; break;
        default                   : wCardType = "Unknown type ?"; break;
      }
            
      wCardUID  = "";
      for (i=0; i<uidlen; i++)
        wCardUID += String.Format("{0:X02}", uid[i]);

      try
      {
        Callback(this, null);
      } catch (Exception)
      {
      
      }
    }

    public void CardAbsent()
    {
      wLastError = SPROX.MI_NOTAGERR;
      wCardType = "";
      wCardUID  = "";
     
      try
      {
        Callback(this, null);
      } catch (Exception)
      {
      
      }
    }

    public void ReaderError(short rc)
    {
      wLastError = rc;

      try
      {
        Callback(this, null);
      } catch (Exception)
      {
      
      }
    }

       
    private short wCheckConnection()
    {
      short rc;
     
      if (wConnected)
      {
        rc = SPROX.Echo(1);
        if (rc == SPROX.MI_OK)
          return rc;
        wConnected = false;
        SPROX.ReaderClose();
      }
      
      rc = SPROX.ReaderOpen("");
      if (rc != SPROX.MI_OK)
        return rc;
        
      if (wStayOFF)
      {
        rc = SPROX.ControlRF(0);
      } else
      {
        rc = SPROX.ControlRF(1);
      }
      if (rc != SPROX.MI_OK)
        return rc;

      wConnected = true;
      wWasConnected = true;
      wFirstConnect = false;
      
      return rc;
    }
    
    private void wTask()
    {
      short rc;
      byte[] uid = new byte[12];
      byte uidlen;
      ushort got_proto;
      
      wRunning = true;
    
      for (;;)
      {       
        if (!wAllowed)
          break;
      
        Thread.Sleep(wInterval);
        
        if (!wAllowed)
          break;
        
        rc = wCheckConnection();
        if (rc != SPROX.MI_OK)
        {                         
          ReaderError(rc);
                  
          /* Connection failed */
          if (wFirstConnect)
            continue;          
          if (wWasConnected && wReconnect)
            continue;
          
          /* Fatal error */
          break;
        }
        
        /* Connection is OK */          
        /* ---------------- */        
        
        if (!wStayOFF)
        {        
          if (wStopRF)
          {
            /* Start RF field */
            rc = SPROX.ControlRF(1);
            if (rc != SPROX.MI_OK)
            {
              ReaderError(rc);
              continue;
            }
          }
        
          /* Lookup */
          got_proto = 0;
          uidlen = 12;
          rc = SPROX.Find(wProtos, ref got_proto, uid, ref uidlen);
        
          /* Got something ? */
          if (rc == SPROX.MI_OK)
          {
            /* Card found */
            CardPresent(got_proto, uid, uidlen);
          } else
          if (rc == SPROX.MI_NOTAGERR)
          {
            /* No card found */
            CardAbsent();
          } else
          {
            /* Another error ? */
            ReaderError(rc);
            continue;
          }
          
          if (wStopRF)
          {
            /* Stop RF field */
            rc = SPROX.ControlRF(0);
            if (rc != SPROX.MI_OK)
            {
              ReaderError(rc);
              continue;
            }
          }
        }
        
        /* Everything is OK ! */
      }

      wRunning = false;

      try 
      {
        SPROX.ControlRF(0);
        SPROX.ReaderClose();      
      } catch (Exception)
      {
      
      }
      
      try
      {
        ReaderError(SPROX.MI_QUIT);
      } catch (Exception)
      {
      
      }      
      
    }
        
  }
}
