using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SpringProxAPI;

namespace SProxPassport
{
  public partial class fReadPassport : Form
  {
    private TwzMRTD wzMRTD;
    private bool ReaderActive = false;

    public fReadPassport()
    {
      InitializeComponent();

#if DEBUG
      this.MinimizeBox = false;
#endif

      /* Instantiate the wzMRTD library */
      wzMRTD = new TwzMRTD();

      wzMRTD.RawOutputPath = "/temp/sproxpassport/";
      wzMRTD.XmlOutputFile = "/temp/sproxpassport/passport.xml";

      wzMRTD.ProgressEvent += new TwzMRTD.TProgressEvent(ReadProgressEvent);

      wzMRTD.SuccessEvent  += new TwzMRTD.TResultEvent(SuccessEvent);
      wzMRTD.FailedEvent   += new TwzMRTD.TResultEvent(FailedEvent);
      wzMRTD.StoppedEvent  += new TwzMRTD.TResultEvent(StoppedEvent);
      wzMRTD.NoReaderEvent += new TwzMRTD.TResultEvent(NoReaderEvent);
    }

    private void btnStop_Click(object sender, EventArgs e)
    {
      btnStop.Enabled = false;
      wzMRTD.StopReading();
    }

    private delegate void TReadProgressCallback(string info, string text, uint pos, uint max);
    private void ReadProgressCallback(string info, string text, uint pos, uint max)
    {
      try
      {
        if ((info != "") && (lbInfo.Text != info))
          lbInfo.Text = info;
        if ((text != "") && (lbText.Text != text))
          lbText.Text = text;
        if (max != 0xFFFFFFFF)
          progressBar.Maximum = (int)max;
        if (pos != 0xFFFFFFFF)
          progressBar.Value = (int)pos;

        Application.DoEvents();
      }
      catch
      {

      }
    }

    private void ReadProgressEvent(string info, string text, uint pos, uint max)
    {
      Invoke(new TReadProgressCallback(ReadProgressCallback), new object[] { info, text, pos, max });
    }

    private delegate void TResultCallback(int result_code);
    private void ResultCallback(int result_code)
    {
      btnStop.Enabled = false;
      lbReading.Visible = false;
      btnRead.Enabled = true;

      if (ReaderActive)
        SPROX.ControlRF(0);

      tmrReader.Enabled = ReaderActive;

      switch (result_code)
      {
        case 1 : /* OK */
          lbReadOK.Visible = true;
          miSee.Enabled = true;
          Application.DoEvents();

          Form f = new fShowPassport();
          f.ShowDialog();
          break;

        case 2: /* KO */
          lbReadKO.Visible = true;
          miSee.Enabled = false;
          break;

        case 3: /* Stopped */
          lbStopped.Visible = true;
          miSee.Enabled = false;
          break;

        case 4: /* No reader */
          ReaderActive = false;
          lbNoReader.Visible = true;
          miSee.Enabled = false;
          break;

        default:
          lbInfo.Text = "Oups !";
          miSee.Enabled = false;
          break;
      }
    }

    private void SuccessEvent()
    {
      Invoke(new TResultCallback(ResultCallback), new object[] { 1 });
    }

    private void FailedEvent()
    {
      Invoke(new TResultCallback(ResultCallback), new object[] { 2 });
    }

    private void StoppedEvent()
    {
      Invoke(new TResultCallback(ResultCallback), new object[] { 3 });
    }

    private void NoReaderEvent()
    {
      Invoke(new TResultCallback(ResultCallback), new object[] { 4 });
    }

    private void btnRead_Click(object sender, EventArgs e)
    {
      tmrReader.Enabled = false;

      btnRead.Enabled = false;
      btnStop.Enabled = true;

      lbReadOK.Visible = false;
      lbReadKO.Visible = false;
      lbNoReader.Visible = false;
      lbStopped.Visible = false;
      lbReading.Visible = true;

      lbInfo.Text = "";
      lbText.Text = "";

      progressBar.Value = 0;

      if (!ReaderActive)
      {
        lbInfo.Text = "Looking for reader...";
        Cursor.Current = Cursors.WaitCursor;
        Application.DoEvents();

        for (int retry = 0; retry < 3; retry++)
        {
          if (SPROX.ReaderOpen(null) == SPROX.MI_OK)
          {
            ReaderActive = true;
            break;
          }
        }
        Cursor.Current = Cursors.Default;
        if (!ReaderActive)
        {
          MessageBox.Show("SpringProx reader not found. Please verify hardware before retrying.");
          ResultCallback(3);
          return;
        }
      }

      try
      {
        System.IO.Directory.Delete(wzMRTD.RawOutputPath, true);
      }
      catch
      {

      }
      try
      {
        System.IO.Directory.CreateDirectory(wzMRTD.RawOutputPath);
      }
      catch
      {

      }

      /* Populate wzMRTD properties */
      wzMRTD.MrzString      = eMrz.Text;

      /* Create a thread to perfom the reading in background */
      System.Threading.Thread thRead = new System.Threading.Thread(new System.Threading.ThreadStart(wzMRTD.ReadPassport));
      thRead.Start();
    }

    private void miAbout_Click(object sender, EventArgs e)
    {
      Form f = new fAbout();
      f.ShowDialog();
    }

    private void fReadPassport_KeyDown(object sender, KeyEventArgs e)
    {
      if ((e.KeyValue == 239))
      {
        e.Handled = true;
        if (btnRead.Enabled)
        {
          btnRead_Click(sender, null);
        } else
        if (btnStop.Enabled)
        {
          btnStop_Click(sender, null);
        }
      }

      if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
      {
        e.Handled = true;
        if (btnRead.Enabled)
        {
          btnRead_Click(sender, null);
        } else
        if (btnStop.Enabled)
        {
          btnStop_Click(sender, null);
        }
      }
      if ((e.KeyCode == System.Windows.Forms.Keys.Up))
      {
        // Bouton  bascule haut
        // Haut
      }
      if ((e.KeyCode == System.Windows.Forms.Keys.Down))
      {
        // Bouton  bascule bas
        // Bas
      }
      if ((e.KeyCode == System.Windows.Forms.Keys.Left))
      {
        // Gauche
      }
      if ((e.KeyCode == System.Windows.Forms.Keys.Right))
      {
        // Droite
      }
      if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
      {
        // Entre
      }

    }

    private void tmrReader_Tick(object sender, EventArgs e)
    {
      tmrReader.Enabled = false;
      ReaderActive = false;
      SPROX.ReaderClose();
    }

    private void fReadPassport_Click(object sender, EventArgs e)
    {
      if (lbReadOK.Visible)
      {
        Form f = new fShowPassport();
        f.ShowDialog();
      }
    }

    private void miSee_Click(object sender, EventArgs e)
    {
      if (lbReadOK.Visible)
      {
        Form f = new fShowPassport();
        f.ShowDialog();
      }
    }

  }
}