using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Runtime.InteropServices;

namespace SProxPassport
{
  public partial class fShowPassport : Form
  {
    public fShowPassport()
    {
      InitializeComponent();

      /* Affiche le curseur d'attente */
      Cursor.Current = Cursors.WaitCursor;

      string XmlFile = "/temp/sproxpassport/passport.xml";

      if (!LoadPassportFromFile(XmlFile))
      {
          Cursor.Current = Cursors.Default;
          MessageBox.Show("Can't display the passport");
      } else
      {
          Cursor.Current = Cursors.Default;
      }
    }

    private void miAbout_Click(object sender, EventArgs e)
    {
      Form f = new fAbout();
      f.ShowDialog();
    }

    private void miReturn_Click(object sender, EventArgs e)
    {
      Close();
    }

    void EmptyDataFields()
    {
        lbType.Text = "";
        lbIssuer.Text = "";
        lbNumber.Text = "";
        lbSurName.Text = "";
        lbGivenName.Text = "";
        lbSex.Text = "";
        lbNationality.Text = "";
        lbDateOfBirth.Text = "";
        lbDateOfExpiry.Text = "";
        lbHeight.Text = "";
        lbColourOfEyes.Text = "";
        lbPlaceOfBirth.Text = "";
        lbAuthority.Text = "";
        lbDateOfIssue.Text = "";
        lbResidence.Text = "";
        pictureID.Visible = false;
    }

    #region Load information from XML file

    string DecodeDate(string str, string fmt)
    {
        if (fmt.Equals("yymmdd") && (str.Length == 6))
        {
            return str.Substring(4, 2) + "/" + str.Substring(2, 2) + "/" + str.Substring(0, 2);
        } else if (fmt.Equals("yyyymmdd") && (str.Length == 8))
        {
            return str.Substring(6, 2) + "/" + str.Substring(4, 2) + "/" + str.Substring(0, 4);
        }
        return str;
    }

    string DecodeDate(string str)
    {
        return DecodeDate(str, "yymmdd");
    }

    string DecodeDateFull(string str)
    {
        return DecodeDate(str, "yyyymmdd");
    }

    bool LoadPassportFromFile(string xmlFileName)
    {
        XmlDocument xmlDoc;
        XmlNode xmlNode;
        string pathName;
        string fileName = null;

        pathName = System.IO.Path.GetFullPath(xmlFileName);
        pathName = System.IO.Path.GetDirectoryName(pathName);
        pathName += System.IO.Path.DirectorySeparatorChar;


        /* Open the XML file */

        try
        {
            xmlDoc = new XmlDocument();
            xmlDoc.Load(xmlFileName);
        } catch
        {
            MessageBox.Show("Failed to load XML file");
            return false;
        }

        /* Erase the fields */
        EmptyDataFields();

        /* Display mandatory fields (from DG1) */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/type");
            lbType.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/issuer");
            lbIssuer.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/number");
            lbNumber.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/surName");
            lbSurName.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/givenName");
            lbGivenName.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/sex");
            lbSex.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/nationality");
            lbNationality.Text = xmlNode.FirstChild.InnerText;

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/dateOfBirth");
            lbDateOfBirth.Text = DecodeDate(xmlNode.FirstChild.InnerText);

            xmlNode = xmlDoc.SelectSingleNode("mrtd/mrz/dateOfExpiry");
            lbDateOfExpiry.Text = DecodeDate(xmlNode.FirstChild.InnerText);
        } catch
        {
            MessageBox.Show("Mandatory item(s) not found in the XML file.");
            return false;
        }

        /* Locate mandatory picture (from DG2) */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/facialBioData/bioTemplate/image");
            fileName = xmlNode.Attributes.GetNamedItem("src").Value;
        } catch
        {
            MessageBox.Show("No picture.");
            return false;
        }

        if (fileName == null)
        {
            MessageBox.Show("No picture.");
            return false;
        }

        if (System.IO.File.Exists(pathName + fileName))
        {
            fileName = pathName + fileName;
        } else
        {
            if (!System.IO.File.Exists(fileName))
            {
                MessageBox.Show("Picture file not found.");
                return false;
            }
        }

        /* Check picture format */
        if (System.IO.Path.GetExtension(fileName).Equals(".jpg"))
        {
            /* Nothing to do ! */
        } else
        {
            if (System.IO.Path.GetExtension(fileName).Equals(".jp2"))
            {
                /* Let's convert the image to a BMP (in the same directory) */
                string bmpFileName = System.IO.Path.ChangeExtension(fileName, ".bmp");
                if (!Convert(fileName, bmpFileName))
                {
                    MessageBox.Show("Failed to decode JPEG2000 image.");
                    return false;
                }
                /* Now work with the BMP */
                fileName = bmpFileName;

            } else
            {
                MessageBox.Show("Unsupported picture format.");
                return false;
            }
        }

        /* Display the picture */
        try
        {
            //PictureID
            Bitmap bmpID;
            //Affichage de l'image
            bmpID = new Bitmap(fileName);

            int height = bmpID.Height;
            int width = bmpID.Width;
            int newHeight;
            int newWidth;

            if (height > width)
            {
              newWidth = (235 * width) / height;
              newHeight = 228;
            } else
            {
              newHeight = (228 * height) / width;
              newWidth = 235;
            }

            pictureID.Image = (Image)bmpID; 
            pictureID.Height = newHeight;
            pictureID.Width = newWidth;
            pictureID.Left = (tabPhoto.Width - newWidth) / 2;
            pictureID.Show();
            pictureID.Visible = true;
        } catch
        {
            MessageBox.Show("Error displaying the picture.");
            return false;
        }

        /* Issuing authority and date of issue : optional, from DG12 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/documentDetails/dateOfIssue");
            lbDateOfIssue.Text = DecodeDateFull(xmlNode.FirstChild.InnerText);
        } catch
        {
        }
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/documentDetails/issuingAuthority");
            lbAuthority.Text = xmlNode.FirstChild.InnerText;
        } catch
        {
        }

        /* Place of birth : optional, from DG11 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/personalDetails/placeOfBirth");
            lbPlaceOfBirth.Text = xmlNode.FirstChild.InnerText;
        } catch
        {
        }
        /* Full birth date : optional, from DG11 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/personalDetails/dateOfBirth");
            lbDateOfBirth.Text = DecodeDateFull(xmlNode.FirstChild.InnerText);
        } catch
        {
        }

        /* Colour of eyes : optional, from DG13 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/optionalDetails/eyeColor");
            lbColourOfEyes.Text = xmlNode.FirstChild.InnerText;
        } catch
        {
        }
        /* Height : optional, from DG13 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/optionalDetails/height");
            lbHeight.Text = xmlNode.FirstChild.InnerText;
        } catch
        {
        }
        /* Residence : optional, from DG13 */
        try
        {
            xmlNode = xmlDoc.SelectSingleNode("mrtd/optionalDetails/address");
            lbResidence.Text = xmlNode.FirstChild.InnerText;
        } catch
        {
        }

        return true;
    }
    #endregion Load information from XML file 

    #region DLL J2KTOBMP
    /*
		 * j2ktobmp
		 * --------
		 * JPEG2000 to BMP convertion, imported from j2ktobmp.dll (based on OpenJPEG)
		 */
    [DllImport("j2ktobmp.dll", CharSet = CharSet.Unicode, EntryPoint = "J2kToBmpW")]
    private static extern bool _ConvertJ2kToBmp(StringBuilder infile, StringBuilder outfile);
    public static bool Convert(string inFileName, string outFileName)
    {
        return _ConvertJ2kToBmp(new StringBuilder(inFileName), new StringBuilder(outFileName));
    }
    #endregion DLL J2KTOBMP

    #region DLL WZMRTD
    public string[] ReadersList;
    public string ReaderName = "SPX";
    public string MrzString = "";
    public string XmlOutputFile = "";
    public string RawOutputPath = "";

    public delegate void TProgressEvent(string info, string trace, uint pos, uint max);
    public delegate void TCompleteEvent(bool success);
    public event TCompleteEvent CompleteEvent;

    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdVersion")]
    public static extern string Version();

    /*
     * Mrtd context
     * ------------
     */
    [DllImport("wzmrtd.dll", CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdAllocCtx")]
    private static extern IntPtr _AllocCtx();
    [DllImport("wzmrtd.dll", CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdFreeCtx")]
    private static extern void _FreeCtx(IntPtr mrtd);

    /*
     * Mrtd errors
     * -----------
     */
    [DllImport("wzmrtd.dll", CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdGetLastError")]
    private static extern int _GetLastError(IntPtr mrtd);
    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdTranslateError")]
    private static extern string _TranslateError(int error);

    /*
     * Mrtd card connect / disconnect
     * ------------------------------
     */
    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdCardConnect")]
    private static extern bool _CardConnect(IntPtr mrtd, string reader, string options);
    [DllImport("wzmrtd.dll", CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdCardDisconnect")]
    private static extern bool _CardDisconnect(IntPtr mrtd);

    /*
     * Read passport
     * -------------
     * Read a contactless passport from a contactless reader...
     */
    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdReadPassport")]
    private static extern bool _ReadPassport(IntPtr mrtd, string mrz);

    /*
     * Save passport to XML file
     * -------------------------
     * Dump the passport to an XML file
     */
    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdSaveToXML")]
    private static extern bool _SaveToXml(IntPtr mrtd, string filename);

    /*
     * Save passport to RAW files
     * --------------------------
     * Dump the passport to a set of RAW files
     */
    [DllImport("wzmrtd.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi, EntryPoint = "MrtdSaveToFiles")]
    private static extern bool _SaveToFiles(IntPtr mrtd, string pathname);

    /*
     * Whole passport read & save sequence
     * -----------------------------------
     */
    public void ReadPassport()
    {
        bool retry = true;
        bool done = false;


        IntPtr ctx = _AllocCtx();

        if (ctx == IntPtr.Zero)
        {
            CompleteEvent(false);
            return;
        }

        while (retry && !done)
        {
            int errno = 0;
            string errmsg = "";

            if (_CardConnect(ctx, ReaderName, ""))
            {
                if (_ReadPassport(ctx, MrzString))
                {
                    if (_SaveToXml(ctx, XmlOutputFile))
                    {
                        /* OK, main job is done */
                        done = true;
                    }
                    else
                    {
                        errmsg = "Failed to save data";
                        errno = _GetLastError(ctx);
                    }
                    if (_SaveToFiles(ctx, RawOutputPath))
                    {
                        /* Don't care... */
                    }
                }
                else
                {
                    errmsg = "Read passport failed";
                    errno = _GetLastError(ctx);
                }
                _CardDisconnect(ctx);
            }
            else
            {
                errmsg = "Read passport error";
                errno = _GetLastError(ctx);
            }

            if (errno != 0)
            {
                retry = (MessageBox.Show(_TranslateError(errno), errmsg, MessageBoxButtons.RetryCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.Retry);
            }
        }

        _FreeCtx(ctx);
        CompleteEvent(done);
    }
    #endregion DLL WZMRTD

  }
}