/*
 * wzPASS - An electronic passport reader software
 * Copyright (c) 2007, Johann Dantant - www.wzpass.net
 *
 * Please read LICENSE.txt for complete informations about the license.
 */

using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace SProxPassport
{  
	public class TwzMRTD
	{
    private bool _Stopped = false;
    private bool _Done    = false;

		public string ReaderName = "SPX";
		public string MrzString = "";
		public string XmlOutputFile = "";
		public string RawOutputPath = "";

    public delegate void TProgressEvent(string info, string text, uint pos, uint max);
    public event TProgressEvent ProgressEvent;

    public delegate void TResultEvent();
    public event TResultEvent SuccessEvent;
    public event TResultEvent FailedEvent;
    public event TResultEvent StoppedEvent;
    public event TResultEvent NoReaderEvent;

		public TwzMRTD()
		{
		}

    private static string AscToStr(byte[] p)
    {
      if (p == null) return null;

      int i = 0;
      string s = "";

      while (p[i] != 0)
      {
        s = s + (char) p[i];
        i++;
      }

      return s;
    }

    private static byte[] StrToAsc(string s)
    {
      if (s == null) return null;

      byte[] p = new byte[1 + s.Length];
      char[] a = s.ToCharArray();
      int i = 0;

      for (i=0; i<a.Length; i++)
      {
        p[i] = (byte) a[i];
      }
      p[a.Length] = 0;

      return p;
    }

    /*
     * Mrtd callbacks
     * -------------- 
     */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdSetCallback")]
    private static extern void _SetCallback(IntPtr dummy);

    /*
		 * Mrtd context
		 * ------------
		 */
		[DllImport("wzmrtd.dll", EntryPoint = "MrtdAllocCtx")]
		private static extern IntPtr _AllocCtx();
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdFreeCtx")]
		private static extern void _FreeCtx(IntPtr mrtd);

		/*
		 * Mrtd errors
		 * -----------
		 */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdGetLastError")]
    private static extern int _GetLastError(IntPtr mrtd);
    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_TranslateError")]
		private static extern void __TranslateError(int error, byte[] dst, int size);
    public static string _TranslateError(int error)
    {
      byte[] dst = new byte[256];
      __TranslateError(error, dst, 256);
      return AscToStr(dst);
    }

		/*
		 * Mrtd card connect / disconnect
		 * ------------------------------
		 */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdCardConnect")]
		private static extern bool __CardConnect(IntPtr mrtd, byte[] reader, byte[] options);
    public static bool _CardConnect(IntPtr mrtd, string reader, string options)
    {
      byte[] pReader  = StrToAsc(reader);
      byte[] pOptions = StrToAsc(options);
      return __CardConnect(mrtd, pReader, pOptions);
    }
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdCardDisconnect")]
		private static extern bool _CardDisconnect(IntPtr mrtd);

		/*
		 * Read passport
		 * -------------
		 * Read a contactless passport from a contactless reader...
		 */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdReadPassport")]
    private static extern bool __ReadPassport(IntPtr mrtd, byte[] mrz);
    private static bool _ReadPassport(IntPtr mrtd, string mrz)
    {
      byte[] pMrz = StrToAsc(mrz);
      return __ReadPassport(mrtd, pMrz);
    }

    [DllImport("wzmrtd.dll", EntryPoint = "MrtdReadPassportEx")]
    private static extern bool __ReadPassportEx(IntPtr mrtd, byte[] mrz, UInt32 want_dgs);
    private static bool _ReadPassportEx(IntPtr mrtd, string mrz, UInt32 want_dgs)
    {
      byte[] pMrz = StrToAsc(mrz);
      return __ReadPassportEx(mrtd, pMrz, want_dgs);
    }

		/*
		 * Save passport to XML file
		 * -------------------------
		 * Dump the passport to an XML file
		 */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdSaveToXML")]
		private static extern bool __SaveToXml(IntPtr mrtd, byte[] filename);
    private static bool _SaveToXml(IntPtr mrtd, string filename)
    {
      byte[] pFilename = StrToAsc(filename);
      return __SaveToXml(mrtd, pFilename);
    }
		
		/*
		 * Save passport to RAW files
		 * --------------------------
		 * Dump the passport to a set of RAW files
		 */
    [DllImport("wzmrtd.dll", EntryPoint = "MrtdSaveToFiles")]
		private static extern bool __SaveToFiles(IntPtr mrtd, byte[] pathname);
    private static bool _SaveToFiles(IntPtr mrtd, string pathname)
    {
      byte[] pPathname = StrToAsc(pathname);
      return __SaveToFiles(mrtd, pPathname);
    }

    /*
     * Specific helpers for PocketPC (CompactFramework doesn't allow callbacks)
     * ------------------------------------------------------------------------
     */
    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_SelectSpringProx")]
    private static extern bool _ConnectReader(IntPtr mtrd);

    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_DeselectSpringProx")]
    private static extern bool _CardRelease(IntPtr mtrd);

    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_ProgressInfo")]
		private static extern void __ProgressInfo(byte[] dst, int size);
    public static string _ProgressInfo()
    {
      byte[] dst = new byte[256];
      __ProgressInfo(dst, 256);
      return AscToStr(dst);
    }

    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_ProgressText")]
		private static extern void __ProgressText(byte[] dst, int size);
    public static string _ProgressText()
    {
      byte[] dst = new byte[256];
      __ProgressText(dst, 256);
      return AscToStr(dst);
    }

    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_ProgressPos")]
		private static extern uint _ProgressPos();

    [DllImport("wzmrtd.dll", EntryPoint = "Mrtd_Ce_ProgressMax")]
		private static extern uint _ProgressMax();

    [DllImport("wzmrtd.dll", EntryPoint = "MrtdStop")]
		private static extern void _Stop();

    /*
     * 
     * 
     */
    public void StopReading()
    {
      _Stop();
      _Stopped = true;
    }
	
		
		/*
		 * Whole passport read & save sequence
		 * -----------------------------------
		 */
	  public void ReadPassport()
		{
      int errno = 0;
      string errmsg = "";

      _Done    = false;
      _Stopped = false;
      _SetCallback(IntPtr.Zero);
		  	 	
  	  IntPtr ctx = _AllocCtx();

  	  if (ctx == IntPtr.Zero)
  	  {
  	    return;
  	  }

      if (!_ConnectReader(ctx))
      {
        _FreeCtx(ctx);
        NoReaderEvent();
        return;
      }

      /* Create a thread to monitor the reading in background */
      System.Threading.Thread thWatch = new System.Threading.Thread(new System.Threading.ThreadStart(_ReadPassportWatcher));
      thWatch.Start();

      while (!_Done)
	 	  {
        errno = 0;

  		 	if (_CardConnect(ctx, "SPX", null))
		 	  {		 	  
		 	  	if (_ReadPassport(ctx, MrzString))
		 	  	{
		 	  		if (_SaveToXml(ctx, XmlOutputFile))
		 	  		{
		 	  			/* OK, main job is done */
              _Done = true;

            } else
		 			  {
		 			    errmsg = "Impossible d'enregistrer les donnes";
		 			  	errno  = _GetLastError(ctx);
  		 			}
  		 			if (_SaveToFiles(ctx, RawOutputPath))
  		 			{
  		 				/* Don't care... */
  		 			}
  		 		} else
  		 		{
  		 		  errmsg = "La lecture a choue";
  		 			errno  = _GetLastError(ctx);
  		 		}
  		 	} else
  		 	{
  		 	  errmsg = "Lecture impossible";
  		 		errno  = _GetLastError(ctx);
  		 	}

        _CardRelease(ctx);

  		 	if (errno != 0)
  		 	{
          if (_Stopped)
            break;

          if (MessageBox.Show(_TranslateError(errno), errmsg, MessageBoxButtons.RetryCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) != DialogResult.Retry)
          {
            /* No more retry */
            break;
          }
          errno = 0;
  		 	}
 	    }

      if (_Done)
      {
        SuccessEvent();
      }
      else
      {
        _Done = true;
        System.Threading.Thread.Sleep(150);
        if (_Stopped)
        {
          StoppedEvent();
        }
        else
        {
          FailedEvent();
        }
      }
 		 	_FreeCtx(ctx);
    }

    private void _ReadPassportWatcher()
    {
      string info, text;
      uint pos, max;

      while (!_Done)
      {       
        info = _ProgressInfo();
        text = _ProgressText();
        pos  = _ProgressPos();
        max  = _ProgressMax();

        ProgressEvent(info, text, pos, max);

        System.Threading.Thread.Sleep(100);
      }
    }
	}
}
