#ifndef __FINGERTK_H__
#define __FINGERTK_H__

#pragma pack(1)

typedef struct
{
  BYTE user_id[8];
  WORD picture_size;
  BYTE finger_size[2];
  BYTE finger_type;
  BYTE birth_yea;
  BYTE birth_mon;
  BYTE birth_day;
  BYTE last_name[32];
  BYTE first_name[32];
} USER_DATA_ST;


typedef struct
{
  USER_DATA_ST user;
  BYTE         fingers[2][170];
} FINGERTK_DATA_ST;


#define RESULT_ERROR          0
#define RESULT_FAILED         1
#define RESULT_SUCCESS        2

#define TAG_NONE       0
#define TAG_MIFARE_1K  1
#define TAG_MIFARE_4K  2
#define TAG_DESFIRE    3

BYTE select_tag(BYTE *tag_uid);
void deselect_tag(BYTE tag_type);
BOOL read_from_tag(FINGERTK_DATA_ST *data);
BOOL read_from_tag_mifare(FINGERTK_DATA_ST *data);
BOOL read_from_tag_desfire(FINGERTK_DATA_ST *data);
BOOL write_to_tag(FINGERTK_DATA_ST *data);
BOOL write_to_tag_mifare(FINGERTK_DATA_ST *data);
BOOL write_to_tag_desfire(FINGERTK_DATA_ST *data);
BYTE verify(FINGERTK_DATA_ST *data);
BYTE enroll(FINGERTK_DATA_ST *data);

/* From files.c */
void save_to_file(FINGERTK_DATA_ST *data);
BOOL load_from_file(FINGERTK_DATA_ST *data);

/* From msoilv.c */
BOOL ilv_loop(BYTE *buffer, WORD *offset, BYTE *tag, WORD *length, BYTE **value);
void ilv_init(BYTE *buffer, WORD *size, BYTE tag);
void ilv_final(BYTE *buffer, WORD *size);
void ilv_add_word(BYTE *buffer, WORD *size, WORD value);
void ilv_add_dword(BYTE *buffer, WORD *size, DWORD value);
void ilv_add_byte(BYTE *buffer, WORD *size, BYTE value);
void ilv_add_data(BYTE *buffer, WORD *size, BYTE *data, WORD length);


void toHEX(TCHAR *text, BYTE *buffer, size_t size);
void toSTR(TCHAR *text, BYTE *buffer, size_t size);
void fromHEX(BYTE *buffer, size_t size, const TCHAR *text);
void fromHEXa(BYTE *buffer, size_t size, const char *text);
void fromSTR(BYTE *buffer, size_t size, const TCHAR *text);


#endif
