#include "stdafx.h"
#include "finger_tk.h"
#include "finger_tkDlg.h"
#include "finger_tkThread.h"
#include "common.h"

#include "hardware/springprox/springprox.h"
#include "cardware/desfire/sprox_desfire.h"
#include "sagem_mso_ilv.h"

#include <stdio.h>
#include <string.h>

#define DESFIRE_APPLICATION_ID   0x00112233
#define DESFIRE_IDENT_FILE_ID    0
#define DESFIRE_PICTURE_FILE_ID  1
#define DESFIRE_FINGER_FILE_ID   2
#define DESFIRE_BALANCE_FILE_ID  3
#define DESFIRE_HISTORY_FILE_ID  4

static const unsigned char DESFIRE_BLANK_KEY[16] =
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00
};
static const unsigned char DESFIRE_KEY_0[16] =
  { 0xB0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x11, 0x22, 0x33
};
static const unsigned char DESFIRE_KEY_1[16] =
  { 0xA0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x11, 0x22, 0x33
};

BOOL read_from_tag_desfire(FINGERTK_DATA_ST * data)
{
  unsigned long length;
  short rc;

  dlg.display(_T("Reading Desfire card..."), NULL);

  /* Try to select our Desfire sample application */
  rc = DesFireCard_SelectApplication(DESFIRE_APPLICATION_ID);
  if (rc != DF_OPERATION_OK)
  {
    dlg.display(NULL, _T("Desfire application not found"));
    return false;
  }

  /* Get authenticated with key 1 */
  rc = DesFireCard_Authenticate(1, DESFIRE_KEY_1);
  if (rc != DF_OPERATION_OK)
  {
    dlg.display(NULL, _T("Desfire authentication failed"));
    return false;
  }

  /* Read the information file. This one is mandatory. It goes directly into our data. */
  rc =
    DesFireCard_ReadData2(DESFIRE_IDENT_FILE_ID, 0, 0, (BYTE *) & data->user,
                          &length);
  if ((rc != DF_OPERATION_OK) || (length != sizeof(data->user)))
  {
    dlg.display(NULL, _T("Error reading user data"));
    return false;
  }

  /* Read the fingerprint file (if some) */
  if (data->user.finger_size[0] || data->user.finger_size[1])
  {
    rc =
      DesFireCard_ReadData2(DESFIRE_FINGER_FILE_ID, 0, 0,
                            (BYTE *) data->fingers, &length);
    if ((rc != DF_OPERATION_OK) || (length != sizeof(data->fingers)))
    {
      dlg.display(NULL, _T("Error reading fingerprints"));
      data->user.finger_size[0] = 0;
      data->user.finger_size[1] = 0;
      return false;
    }
  }

  /* Be optimistic regarding the actual content of the file... */
  dlg.display(_T("Desfire tag read"), _T(""));
  return true;
}

BOOL format_tag_desfire()
{
  short rc;

  dlg.display(_T("Formating Desfire tag..."), _T(""));

  /* Select root application in the Desfire card */
  rc = DesFireCard_SelectApplication(0);
  if (rc != MI_OK)
    return false;

  /* Perform 3-DES authentication with card's master key (must be blank !) */
  rc = DesFireCard_Authenticate(0, DESFIRE_BLANK_KEY);
  if (rc != MI_OK)
    return false;

  /* Create our application. Application has 2 keys. We don't care of key_settings here. */
  rc = DesFireCard_CreateApplication(DESFIRE_APPLICATION_ID, 0xE3, 2);
  if (rc != MI_OK)
    return false;

  /* Select the newly created application */
  rc = DesFireCard_SelectApplication(DESFIRE_APPLICATION_ID);
  if (rc != MI_OK)
    return false;

  /* Get authenticated with blank key 0 */
  rc = DesFireCard_Authenticate(0, DESFIRE_BLANK_KEY);
  if (rc != MI_OK)
    return false;

  /* Create the files */
  rc = DesFireCard_CreateStdDataFile(DESFIRE_IDENT_FILE_ID, 0, 0x1F00, 80);
  if (rc != MI_OK)
    return false;
  rc =
    DesFireCard_CreateStdDataFile(DESFIRE_PICTURE_FILE_ID, 0, 0x1F00, 3072);
  if (rc != MI_OK)
    return false;
  rc = DesFireCard_CreateStdDataFile(DESFIRE_FINGER_FILE_ID, 3, 0x1F00, 340);
  if (rc != MI_OK)
    return false;
  rc =
    DesFireCard_CreateValueFile(DESFIRE_BALANCE_FILE_ID, 2, 0x1F00, 0, 100000,
                                0, 0);
  if (rc != MI_OK)
    return false;
  rc =
    DesFireCard_CreateCyclicRecordFile(DESFIRE_HISTORY_FILE_ID, 0, 0xEEEE, 8,
                                       4);
  if (rc != MI_OK)
    return false;

  rc = DesFireCard_ChangeKey(0, DESFIRE_KEY_0, NULL);
  if (rc != MI_OK)
    return false;
  rc = DesFireCard_Authenticate(1, DESFIRE_BLANK_KEY);
  if (rc != MI_OK)
    return false;
  rc = DesFireCard_ChangeKey(1, DESFIRE_KEY_1, NULL);
  if (rc != MI_OK)
    return false;

  return true;
}

BOOL write_to_tag_desfire(FINGERTK_DATA_ST * data)
{
  /* Select the application, or create it if not found */
  if (DesFireCard_SelectApplication(DESFIRE_APPLICATION_ID) != MI_OK)
    if (!format_tag_desfire())
    {
      dlg.display(NULL, _T("Failed to format the Desfire tag"));
      return false;
    }

  dlg.display(_T("Writing Desfire tag..."), _T(""));

  /* Get authenticated with key 0 for writing */
  if (DesFireCard_Authenticate(0, DESFIRE_KEY_0) != MI_OK)
  {
    dlg.display(NULL, _T("Desfire authentication failed"));
    return false;
  }

  /* Write the information block */
  if (DesFireCard_WriteData2
      (DESFIRE_IDENT_FILE_ID, 0, sizeof(data->user),
       (BYTE *) & data->user) != MI_OK)
  {
    dlg.display(NULL, _T("Error writing user data"));
    return false;
  }

  /* Write the fingerprints block */
  if (data->user.finger_size[0] || data->user.finger_size[1])
  {
    if (DesFireCard_WriteData2
        (DESFIRE_FINGER_FILE_ID, 0, sizeof(data->fingers),
         (BYTE *) data->fingers) != MI_OK)
    {
      dlg.display(NULL, _T("Error writing fingerprints"));
      return false;
    }
  }

  return true;
}
