#include "stdafx.h"
#include "finger_tk.h"
#include "finger_tkDlg.h"
#include "finger_tkThread.h"
#include "common.h"

#include "hardware/springprox/springprox.h"
#include "sagem_mso_ilv.h"

void enroll_async_action(BYTE code);
void enroll_async_info(BYTE finger_no, BYTE finger_count, BYTE capture_no,
                       BYTE capture_count);

/* SAGEM MSO enrollment command (2 fingers) */

/* ---------------------------------------- */
BYTE enroll(FINGERTK_DATA_ST * data)
{
  BYTE buffer[1024];
  BYTE async;
  WORD size;
  BYTE tag;
  WORD offset, length;
  BYTE *value;
  BYTE idx;
  SWORD rc;

  /* Build the ILV enroll command               */
  /* (SEE SAGEM DOCUMENTATION FOR EXPLANATIONS) */
  /* ------------------------------------------ */

  memset(buffer, 0, sizeof(buffer));
  length = 0;

  /* Initialize ILV */
  ilv_init(buffer, &length, ILV_ENROLL);

  /* Build ILV */
  ilv_add_byte(buffer, &length, 0); /* Database identifier = 0 */
  ilv_add_word(buffer, &length, 60);  /* Timeout = 60 seconds */
  ilv_add_byte(buffer, &length, 0); /* Acquisition quality threshold = default (0) */
  ilv_add_byte(buffer, &length, 0); /* Enrollment type = default (0) */
  ilv_add_byte(buffer, &length, 2); /* We want to enroll 2 fingers */
  ilv_add_byte(buffer, &length, 0); /* Save record into local database = NO (0) */
  ilv_add_byte(buffer, &length, 1); /* Export minutiae with its default size = YES (1) */

  /* We want to receive asynchronous events */
  ilv_add_byte(buffer, &length, ID_ASYNCHRONOUS_EVENT);
  ilv_add_word(buffer, &length, 4);
  ilv_add_dword(buffer, &length, 0x00000005);
  /* Bit field = 0x00000005 : action messages (move left, press harder...) */
  /* and information messages (finger X/X, capture X/X) will be sent       */

  /* Write length in the ILV */
  ilv_final(buffer, &length);

  /* Send the command to the MSO and process its responses (including asynchronous messages) */
  /* --------------------------------------------------------------------------------------- */
  async = FALSE;
  for (;;)
  {
    size = sizeof(buffer);

    /* NOTE : the timeout of the exchange must be a little greater than the timeout offered  */
    /*        to the MSO, overwhise the library will return an error code before the MSO has */
    /*        finished is process                                                            */
    if (async)
      rc = SPROX_MSO_Exchange(NULL, 0, buffer, &size, 65000, &async);
    else
      rc = SPROX_MSO_Exchange(buffer, length, buffer, &size, 65000, &async);

    if (rc != 0)
      return RESULT_ERROR;

    if (async)
    {
      /* Easy processing of asynchronous message, no error control */
      switch (buffer[4])
      {
        case 0x01:
          enroll_async_action(buffer[7]);
          break;
        case 0x04:
          enroll_async_info(buffer[7], buffer[8], buffer[9], buffer[10]);
          break;
      }

      /* NOTE : we MUST go back to MSO_Exchange AS SOON AS POSSIBLE because if we were to  */
      /*        miss any message (asynchronous or final result), the reader will report a  */
      /*        protocol error and abort the process                                       */
    } else
    {
      break;                    /* Last message -> end of the loop */
    }
  }

  /* 
   * Reply format is :
   *  0      : reply ID
   *  1 -> 2 : reply length
   *  3      : request status
   *  4      : enroll status
   *  5 -> 8 : user database index
   *  9      : first finger
   *  ...    : second finger
   */

  /* Check returned reply and status */
  /* ------------------------------- */

  if (buffer[0] != ILV_ENROLL)
  {
    dlg.display(_T(""), _T("Wrong MSO reply"));
    return RESULT_ERROR;
  }
  if (buffer[3] != 0x00)
  {
    dlg.display(_T(""), _T("Wrong MSO status"));
    return RESULT_ERROR;
  }
  if (buffer[4] != 0x00)
  {
    dlg.display(_T(""), _T("Wrong MSO result"));
    return RESULT_FAILED;
  }

  /* Decode returned data */
  /* -------------------- */

  offset = 9;
  idx = 0;
  while ((offset < size) && ilv_loop(buffer, &offset, &tag, &length, &value))
  {
    if ((tag != ID_PKCOMP)      /* PKCOMP minutiae */
        && (tag != ID_PKMAT))   /* PKMAT minutiae */
    {
      /* Skip */
      continue;
    }

    if (idx < 2)
    {
      /* Finger 1 or 2 */
      if (length > sizeof(data->fingers) / 2)
      {
        continue;
      }
      if (idx == 0)
      {
        data->user.finger_type = tag;
        memcpy(data->fingers[0], value, length);
        data->user.finger_size[0] = (BYTE) length;
      } else if (tag == data->user.finger_type)
      {
        memcpy(data->fingers[1], value, length);
        data->user.finger_size[1] = (BYTE) length;
      }
      idx++;
    } else
    {
      /* Silently ignore the finger */
    }
  }

  return RESULT_SUCCESS;
}

void enroll_async_action(BYTE code)
{
  switch (code)
  {
    case 0:
      dlg.display(NULL, _T(""));
      break;
    case 1:
      dlg.display(NULL, _T("Move up"));
      break;
    case 2:
      dlg.display(NULL, _T("Move down"));
      break;
    case 3:
      dlg.display(NULL, _T("Move left"));
      break;
    case 4:
      dlg.display(NULL, _T("Move right"));
      break;
    case 5:
      dlg.display(NULL, _T("Press harder"));
      break;
    case 6:
      dlg.display(NULL, _T("Move finger"));
      break;
    case 7:
      //dlg.display(NULL, _T("Remove finger"));
      break;
    case 8:
      dlg.display(NULL, _T("Finger OK"));
      break;
  }
}

void enroll_async_info(BYTE finger_no, BYTE finger_count, BYTE capture_no,
                       BYTE capture_count)
{
  TCHAR buffer[40];

  _stprintf(buffer, _T("Finger %d/%d, capture %d/%d"), finger_no,
            finger_count, capture_no, capture_count);
  dlg.display(buffer, NULL);
}
