#include "stdafx.h"
#include "finger_tk.h"
#include "finger_tkDlg.h"
#include "finger_tkThread.h"
#include "common.h"

#include "hardware/springprox/springprox.h"
#include "cardware/desfire/sprox_desfire.h"
#include "sagem_mso_ilv.h"

#include <stdio.h>
#include <string.h>


WORD swap_word(WORD v)
{
  BYTE t[2];
  WORD w;

  t[0] = (BYTE) (v / 0x0100);
  t[1] = (BYTE) (v % 0x0100);
  memcpy(&w, t, 2);
  return w;
}

BOOL read_from_tag(FINGERTK_DATA_ST * data)
{
  BYTE tag_type;

  dlg.display(_T("Looking for a tag..."), _T(""));
  memset(data, 0, sizeof(FINGERTK_DATA_ST));

  tag_type = select_tag(NULL);

  if (tag_type == TAG_DESFIRE)
    return read_from_tag_desfire(data);

  if ((tag_type == TAG_MIFARE_1K) || (tag_type == TAG_MIFARE_4K))
    return read_from_tag_mifare(data);

  return FALSE;
}


BOOL write_to_tag(FINGERTK_DATA_ST * data)
{
  BYTE tag_type;

  dlg.display(_T("Looking for a tag..."), _T(""));

  tag_type = select_tag(NULL);

  if (tag_type == TAG_DESFIRE)
  {
    return write_to_tag_desfire(data);
  } else if ((tag_type == TAG_MIFARE_1K) || (tag_type == TAG_MIFARE_4K))
  {
    return write_to_tag_mifare(data);
  } else
    return FALSE;
}



/*
 * Tag manipulation functions
 * --------------------------
 */

BYTE select_tag(BYTE * tag_uid)
{
  BYTE snr[12];
  BYTE snrlen = 12;
  BYTE atq[2];
  BYTE sak[1];
  BYTE ats[32];
  BYTE atslen = 32;
  SWORD rc;

  /* ISO 14443-3 activation, with T=CL compliance check */
  rc = SPROX_TclA_ActivateAny(atq, snr, &snrlen, sak);

  if (tag_uid != NULL)
    memcpy(tag_uid, snr, snrlen);

  if (rc == TCL_NOTAGERR)
  {
    /* Not a T=CL tag, maybe Mifare ? */
    rc = SPROX_MifStSelectAny(snr, atq, sak);
    if (rc != MI_OK)
    {
      dlg.display(NULL, _T("Mifare activation failed"));
      return TAG_NONE;
    }
    if (atq[1] == 0x00)
    {
      if (atq[0] == 0x04)       /* Mifare 1k */
      {
        dlg.display(NULL, _T("Found Mifare 1k tag"));
        sprintf(TagInfo, "Mifare 1k %02X%02X%02X%02X", snr[3], snr[2], snr[1],
                snr[0]);
        return TAG_MIFARE_1K;
      }
      if (atq[0] == 0x02)       /* Mifare 4k */
      {
        dlg.display(NULL, _T("Found Mifare 4k tag"));
        sprintf(TagInfo, "Mifare 4k %02X%02X%02X%02X", snr[3], snr[2], snr[1],
                snr[0]);
        return TAG_MIFARE_4K;
      }
    }
    /* Not supported */
    SPROX_TclA_Halt();
    dlg.display(NULL, _T("Unsupported tag"));
    return TAG_NONE;
  } else if (rc != MI_OK)
  {
    /* No tag ? */
    return TAG_NONE;
  }

  if ((atq[1] == 0x03) && (atq[0] == 0x44))
  {
    /* This seems to be a Desfire... */
    dlg.display(NULL, _T("Found Desfire tag"));
    rc = SPROX_TclA_GetAts(0xFF, ats, &atslen);
    if (rc == MI_OK)
    {
      /* Desfire activated */
      sprintf(TagInfo, "Desfire %02X%02X%02X%02X%02X%02X%02X", snr[0], snr[1],
              snr[2], snr[3], snr[4], snr[5], snr[6]);
      return TAG_DESFIRE;
    } else
    {
      dlg.display(NULL, _T("Desfire activation failed"));
    }
  } else
  {
    dlg.display(NULL, _T("Unsupported tag"));
  }

  /* Not supported, or no tag activated */
  SPROX_TclA_Halt();
  return TAG_NONE;
}


void deselect_tag(BYTE tag_type)
{
  if (tag_type == TAG_NONE)
    return;
  if (tag_type == TAG_DESFIRE)
    SPROX_TclA_Deselect(0xFF);
  SPROX_TclA_Halt();
}



void toHEX(TCHAR * text, BYTE * buffer, size_t size)
{
  size_t i;

  for (i = 0; i < size; i++)
  {
    _stprintf(&text[2 * i], _T("%02X"), buffer[i]);
  }
}

void toSTR(TCHAR * text, BYTE * buffer, size_t size)
{
  size_t i;

  for (i = 0; i < size; i++)
  {
    text[i] = buffer[i];
    if (buffer[i] == 0)
      break;
  }
}

void fromHEX(BYTE * buffer, size_t size, const TCHAR * text)
{
  size_t i;

  for (i = 0; i < size; i++)
    buffer[i] = 0;

  for (i = 0; i < _tcslen(text); i++)
  {
    if (i >= 2 * size)
      break;

    char c = (char) text[i];

    BYTE b = 0;

    if ((c >= '0') && (c <= '9'))
      b = (BYTE) (c - '0');
    else if ((c >= 'A') && (c <= 'F'))
      b = (BYTE) (c + 10 - 'A');
    else if ((c >= 'a') && (c <= 'f'))
      b = (BYTE) (c + 10 - 'a');

    if ((i % 2) != 0)
    {
      buffer[i / 2] |= b;
    } else
    {
      buffer[i / 2] = (BYTE) (b << 4);
    }
  }
}

void fromHEXa(BYTE * buffer, size_t size, const char *text)
{
  size_t i;

  for (i = 0; i < size; i++)
    buffer[i] = 0;

  for (i = 0; i < strlen(text); i++)
  {
    if (i >= 2 * size)
      break;

    char c = (char) text[i];

    BYTE b = 0;

    if ((c >= '0') && (c <= '9'))
      b = (BYTE) (c - '0');
    else if ((c >= 'A') && (c <= 'F'))
      b = (BYTE) (c + 10 - 'A');
    else if ((c >= 'a') && (c <= 'f'))
      b = (BYTE) (c + 10 - 'a');

    if ((i % 2) != 0)
    {
      buffer[i / 2] |= b;
    } else
    {
      buffer[i / 2] = (BYTE) (b << 4);
    }
  }
}

void fromSTR(BYTE * buffer, size_t size, const TCHAR * text)
{
  size_t i;

  for (i = 0; i < size; i++)
    buffer[i] = 0;

  for (i = 0; i < _tcslen(text); i++)
  {
    if (i >= size)
      break;

    buffer[i] = (BYTE) text[i];
  }
}
