#include "stdafx.h"
#include "finger_tk.h"
#include "finger_tkDlg.h"
#include "finger_tkThread.h"
#include "common.h"

#include "hardware/springprox/springprox.h"
#include "sagem_mso_ilv.h"

#include <stdio.h>
#include <string.h>

void verify_async_action(BYTE code);

/* SAGEM MSO verify command (1 or 2 fingers) */

/* ----------------------------------------- */

/* return : 0 if verification has not been done (error ?)   */

/*          1 if verification failed (not the right user ?) */

/*          2 if verification succeeded (the right user !)  */
BYTE verify(FINGERTK_DATA_ST * data)
{
  BYTE buffer[1024];
  BYTE async;
  WORD size;
  WORD length;
  SWORD rc;

  dlg.display(_T("Verifiying..."), _T(""));


  /* Build the ILV verify command               */
  /* (SEE SAGEM DOCUMENTATION FOR EXPLANATIONS) */
  /* ------------------------------------------ */

  memset(buffer, 0, sizeof(buffer));
  length = 0;

  /* Initialize ILV */
  ilv_init(buffer, &length, ILV_VERIFY);

  /* Build ILV */
  ilv_add_word(buffer, &length, 30);  /* Timeout = 60 seconds */
  ilv_add_word(buffer, &length, 5); /* Matching threshold = 5 (recommended value) */
  ilv_add_byte(buffer, &length, 0); /* Acquisition quality threshold = default (0) */

  /* Add fingers */
  for (int idx = 0; idx < 2; idx++)
  {
    if (data->user.finger_size[idx])
    {
      ilv_add_byte(buffer, &length, data->user.finger_type);
      ilv_add_word(buffer, &length, data->user.finger_size[idx]);
      ilv_add_data(buffer, &length, data->fingers[idx],
                   data->user.finger_size[idx]);
    }
  }

  /* We want to receive asynchronous events */
  ilv_add_byte(buffer, &length, ID_ASYNCHRONOUS_EVENT);
  ilv_add_word(buffer, &length, 4);
  ilv_add_dword(buffer, &length, 0x00000001);

  /* Write length in the ILV */
  ilv_final(buffer, &length);

  /* Send the command to the MSO and process its responses (including asynchronous messages) */
  /* --------------------------------------------------------------------------------------- */
  async = FALSE;
  for (;;)
  {
    size = sizeof(buffer);

    /* NOTE : the timeout of the exchange must be a little greater than the timeout offered  */
    /*        to the MSO, overwhise the library will return an error code before the MSO has */
    /*        finished is process                                                            */
    if (async)
      rc = SPROX_MSO_Exchange(NULL, 0, buffer, &size, 35000, &async);
    else
      rc = SPROX_MSO_Exchange(buffer, length, buffer, &size, 35000, &async);

    if (rc != 0)
      return RESULT_ERROR;      /* Error */

    if (async)
    {
      /* Easy processing of asynchronous message, no error control */
      switch (buffer[4])
      {
        case 0x01:
          verify_async_action(buffer[7]);
          break;
      }

      /* NOTE : we MUST go back to MSO_Exchange AS SOON AS POSSIBLE because if we were to  */
      /*        miss any message (asynchronous or final result), the reader will report a  */
      /*        protocol error and abort the process                                       */
    } else
    {
      break;                    /* Last message -> end of the loop */
    }
  }

  /* 
   * Reply format is :
   *  0      : reply ID
   *  1 -> 2 : reply length
   *  3      : request status
   *  4      : matching result
   */

  /* Check returned reply and status */
  /* ------------------------------- */

  if (buffer[0] != ILV_VERIFY)
  {
    dlg.display(_T(""), _T("Wrong MSO reply"));
    return RESULT_ERROR;
  }
  if (buffer[3] != 0x00)
  {
    dlg.display(_T(""), _T("Wrong MSO status"));
    return RESULT_ERROR;
  }

  /* Check matching result */
  /* --------------------- */

  switch (buffer[4])
  {
    case ILVSTS_HIT:           /* Verify -> OK ! */
      return RESULT_SUCCESS;

    case ILVSTS_NO_HIT:        /* Verify -> KO ! */
      return RESULT_FAILED;
  }

  dlg.display(_T(""), _T("Wrong MSO result"));
  return RESULT_ERROR;
}

void verify_async_action(BYTE code)
{
  switch (code)
  {
    case 0:
      dlg.display(NULL, _T(""));
      break;
    case 1:
      dlg.display(NULL, _T("Move up"));
      break;
    case 2:
      dlg.display(NULL, _T("Move down"));
      break;
    case 3:
      dlg.display(NULL, _T("Move left"));
      break;
    case 4:
      dlg.display(NULL, _T("Move right"));
      break;
    case 5:
      dlg.display(NULL, _T("Press harder"));
      break;
    case 6:
      dlg.display(NULL, _T("Move finger"));
      break;
    case 7:
      dlg.display(NULL, _T("Remove finger"));
      break;
    case 8:
      dlg.display(NULL, _T("Finger OK"));
      break;
  }
}
